/****************************************************************
$Module       : XOLECTL (xolectl.h)
$Library      : 
$Class        : CExtOleControl
$Derivation   : COleControl
$Purpose      : To provide extended functionality for a 
                COleControl class.

$Usage        : Per Property Browsing
                ---------------------
                Use the helper macros in your derived class to get
                a free implementation of the Per Property Browsing 
                interface. 
                
                In the derived classes header file, add the line:
                  DECLARE_PROPERTY_LIST                  

                In the derived classes implementation file, add 
                similar lines

                  BEGIN_PROPERTY_LIST(Derived Class Name)
                     BEGIN_PROPERTY_ENUM(
                        ClassName, memberName, property1Dispid)
                        PROPERTY_VALUE(0, IDS_STRING1)
                        PROPERTY_VALUE(1, IDS_STRING2)
                        PROPERTY_VALUE(2, IDS_STRING3)
                     END_PROPERTY_ENUM

                     BEGIN_PROPERTY_ENUM_NODISPLAY(property2Dispid)
                        PROPERTY_VALUE(1, IDS_STRING4)
                        PROPERTY_VALUE(2, IDS_STRING5)
                        PROPERTY_VALUE(4, IDS_STRING6)
                     END_PROPERTY_ENUM
                  END_PROPERTY_LIST

                These macros will setup a table that this base 
                class can use  to implement the 
                OnGetPredefinedStrings, OnGetPredefinedValue
                and OnGetDisplayString methods.

$See Also     : xolepp.h for description of CExtOlePropertyPage 
                class which should be used in conjunction with the 
                to perform data transfer with the enumerated 
                properties.
****************************************************************/

#ifndef _XOLECTL_H
#define _XOLECTL_H

/*****************************************************************
   Forward References
******************************************************************/
struct SEnumPropertyValue;
struct SEnumPropertyMap;


/*****************************************************************
   CExtOleControl class definition
******************************************************************/
class CExtOleControl : public COleControl
{
/*----------------------------
   Construction/Destruction
----------------------------*/
public:
   CExtOleControl();
   ~CExtOleControl();

/*----------------------------
   Implementation
----------------------------*/
protected:
   virtual const SEnumPropertyMap* GetEnumeratedPropertyList();
   virtual const SEnumPropertyMap* GetEnumeratedPropertyMap(
      DISPID dispid);

   // Virtual Overrides
public:
   virtual BOOL OnGetPredefinedStrings( DISPID dispid, 
      CStringArray* pStringArray, CDWordArray* pCookieArray);

   virtual BOOL OnGetPredefinedValue(DISPID dispid, 
      DWORD dwCookie, VARIANT* lpvarOut);

   virtual BOOL OnGetDisplayString(DISPID dispid, 
      CString& strValue);

};

/*****************************************************************
   SEnumPropertyValue definition

   helper structure that stores an individual possible value for a 
   property and the string used to describe that value
******************************************************************/
struct SEnumPropertyValue
{
   long  m_nValue;
   UINT  m_idString;

   SEnumPropertyValue(long value, UINT idString);
};

/*****************************************************************
   SEnumPropertyMap definition

   helper structure used to map all possible values for a property
   and their corresponding string descriptions
******************************************************************/
struct SEnumPropertyMap
{
   DISPID m_dispid;

   long   m_nOffset;        // offset into class for the member
                            // member variable that corresponds to
                            // this property

   long   m_numBytes;       // size of the corresponding member
                            // variable

   CMapPtrToPtr* m_pMap;

   SEnumPropertyMap(DISPID dispid, 
      long nOffset, long numBytes, ...);
   virtual ~SEnumPropertyMap();
};

/*****************************************************************
   Property Map macros used to create automatic support for 
   per-property-browsing.  See usage in module header above!
******************************************************************/
#define DECLARE_PROPERTY_LIST                                     \
   public:                                                        \
      static const SEnumPropertyMap m_arEnumPropertyMap[];        \
      virtual const SEnumPropertyMap* GetEnumeratedPropertyList() \
         { return m_arEnumPropertyMap; }


#define BEGIN_PROPERTY_LIST(className)   \
   const SEnumPropertyMap className::m_arEnumPropertyMap[] = {

#define END_PROPERTY_LIST  \
   {SEnumPropertyMap(0,0,0)} };

#define BEGIN_PROPERTY_ENUM(className, member, dispid)     \
   { SEnumPropertyMap(dispid, offsetof(className,member),  \
        membsizeof(className, member), 

#define BEGIN_PROPERTY_ENUM_NODISPLAY(dispid) \
   { SEnumPropertyMap(dispid, 0, 0, 

#define PROPERTY_VALUE(value, idString) \
   new SEnumPropertyValue(value, idString), 

#define END_PROPERTY_ENUM \
   new SEnumPropertyValue(0,0) ) },

#endif